import random
import string

from django.conf import settings

from .tasks import (
    create_wordpress_user as create_wordpress_user_task,
    update_wordpress_user as update_wordpress_user_task,
    update_wordpress_role as update_wordpress_role_task
)


def create_wordpress_user(user):
    # generate a random email if no email is provided
    if user.email:
        email = user.email
    else:
        random_string = ''.join(random.choice(string.ascii_lowercase) for _ in range(8))
        email = random_string + '@example.com'

    if not settings.ASYNC:
        create_wordpress_user_task.apply((user.username, email, user.first_name, user.last_name), throw=True)
    else:
        create_wordpress_user_task.apply_async((user.username, user.email, user.first_name, user.last_name))


def update_wordpress_user(user):
    if not settings.ASYNC:
        update_wordpress_user_task.apply((user.username, user.email, user.first_name, user.last_name), throw=True)
    else:
        update_wordpress_user_task.apply_async((user.username, user.email, user.first_name, user.last_name))


def update_wordpress_role(user):
    if user.is_superuser:
        wordpress_role = 'administrator'
    elif user.groups.filter(name='wordpress_admin').exists():
        wordpress_role = 'administrator'
    elif user.groups.filter(name='wordpress_editor').exists():
        wordpress_role = 'editor'
    else:
        wordpress_role = 'subscriber'

    if not settings.ASYNC:
        update_wordpress_role_task.apply((user.username, wordpress_role), throw=True)
    else:
        update_wordpress_role_task.apply_async((user.username, wordpress_role))



import os

from .models import Directory


def get_file(user, file_path):
    # append 'index.html' when the file_path is a directory
    if file_path.endswith('/'):
        file_path += 'index.html'

    # get directories this user has access to
    directories = Directory.objects.filter_by_access_level(user)

    results = set()
    for directory in directories:
        # normalize the file path so that /a/b/c and b/c/d/e become /a/b/c and d/e
        normalized_file_path = normalize_file_path(directory.path, file_path)

        # join directory_path and file_path so that it becomes /a/b/c/d/e
        absolute_file_path = os.path.join(directory.path, normalized_file_path)

        # check if absolute_file_path is actually a file
        if os.path.isfile(absolute_file_path):
            results.add(absolute_file_path)

    # check if we found the file more than once
    if len(results) > 1:
        raise Exception('More than one file found in %s.get_file().' % __name__)
    elif len(results) == 1:
        return results.pop()
    else:
        return None


def search_file(user, file_path):
    # get directories this user has access to
    directories = Directory.objects.filter_by_access_level(user)

    results = set()
    for directory in directories:
        for directory_path, dirs, files in os.walk(directory.path):
            # normalize the file path so that /a/b/c and b/c/d/e become /a/b/c and d/e
            normalized_file_path = normalize_file_path(directory_path, file_path)

            # join directory_path and file_path so that it becomes /a/b/c/d/e
            absolute_file_path = os.path.join(directory_path, normalized_file_path)

            # check if absolute_file_path is actually a file
            if os.path.isfile(absolute_file_path):
                results.add(absolute_file_path)

    # check if we found the file more than once
    if len(results) > 1:
        raise Exception('More than one file found in %s.get_file().' % __name__)
    elif len(results) == 1:
        return results.pop()
    else:
        return None


def normalize_file_path(directory_path, file_path):

    directory_path_tokens = os.path.normpath(directory_path).split(os.path.sep)
    file_path_tokens = os.path.normpath(file_path).split(os.path.sep)

    match = 0
    for i in range(len(file_path_tokens)):
        if file_path_tokens[:i] == directory_path_tokens[-i:]:
            match = i

    return os.path.join(*file_path_tokens[match:])



import imp
import os

from django.utils.translation import ugettext_lazy as _

CONFIG_DIR = imp.find_module('config')[1]
BASE_DIR = os.path.dirname(CONFIG_DIR)
DAIQUIRI_APP = os.path.basename(BASE_DIR).replace('-', '_')

DEBUG = False

SITE_ID = 1

HTTPS = False

SECRET_KEY = 'this is not a very secret key'

ALLOWED_HOSTS = ['localhost']

INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'config.urls'

WSGI_APPLICATION = 'config.wsgi.application'

DATABASE_ROUTERS = ['daiquiri.tap.routers.TapRouter', 'daiquiri.core.routers.DataRouter']

DJANGO_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
]

ADDITIONAL_APPS = [
    'rest_framework',
    'rest_framework.authtoken',
    'django_filters',
    'django_extensions',
    'markdown',
    'compressor',
    'widget_tweaks',
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'rules'
]

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sites.middleware.CurrentSiteMiddleware'
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates/')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django_settings_export.settings_export',
            ],
        },
    },
]

COMPRESS_PRECOMPILERS = (
    ('text/x-scss', 'django_libsass.SassCompiler'),
)

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale/'),
)

LANGUAGES = (
    ('en', _('English')),
)

USE_I18N = True

USE_L10N = True

USE_TZ = True

LOGIN_URL = '/accounts/login/'
LOGIN_REDIRECT_URL = '/'
LOGOUT_URL = '/accounts/logout/'

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media_root/')

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static_root/')

STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'vendor/'),
    os.path.join(BASE_DIR, 'static/'),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder'
)

FIXTURE_DIRS = (
    os.path.join(BASE_DIR, 'fixtures'),
)

AUTHENTICATION_BACKENDS = (
    'rules.permissions.ObjectPermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
    'allauth.account.auth_backends.AuthenticationBackend',
)

ACCOUNT_ADAPTER = 'daiquiri.auth.adapter.DaiquiriAccountAdapter'
ACCOUNT_SIGNUP_FORM_CLASS = 'daiquiri.auth.forms.SignupForm'
ACCOUNT_USER_DISPLAY = 'daiquiri.auth.utils.get_full_name'
ACCOUNT_EMAIL_MAX_LENGTH = 190
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_CONFIRMATION_EXPIRE_DAYS = 7
ACCOUNT_EMAIL_VERIFICATION = 'mandatory'
ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = True

SOCIALACCOUNT_ADAPTER = 'daiquiri.auth.adapter.DaiquiriSocialAccountAdapter'

SETTINGS_EXPORT = [
    'LOGIN_URL',
    'LOGOUT_URL',
    'AUTH_WORKFLOW',
    'QUERY_DROPDOWNS',
    'QUERY_FORMS',
    'QUERY_DOWNLOAD_FORMATS'
]

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
EMAIL_FROM = 'info@example.com'

SENDFILE_BACKEND = 'sendfile.backends.simple'

LOGGING_DIR = os.path.join(BASE_DIR, 'log')



import sys

from setuptools import setup, find_packages

from daiquiri import __title__, __email__, __version__, __author__, __license__

setup(
    name=__title__,
    version=__version__,
    author=__author__,
    author_email=__email__,
    maintainer=__author__,
    maintainer_email=__email__,
    license=__license__,
    url='https://github.com/aipescience/django-daiquiri',
    description=u'Daiquiri is a framework for the publication of scientific databases.',
    long_description=open('README.rst').read(),
    install_requires=[
        'Django>=1.11',
        'djangorestframework>=3.6.2',
        'drf-extensions>=0.3.1',
        'django-extensions>=1.8.1',
        'django-allauth>=0.32.0',
        'django-filter>=1.0.4',
        'django-widget-tweaks>=1.4.1',
        'django-compressor>=2.1.1',
        'django-libsass>=0.7',
        'django-settings-export>=1.2.1',
        'django-sendfile>=0.3.11',
        'django-ipware>=1.1.6',
        'celery>=4.1.0',
        'redis>=2.10.5',
        'rules>=1.2',
        'jsonfield==1.0.0',
        'Markdown>=2.6.8',
        'iso8601>=0.1.11',
        'lxml>=3.7.3',
        'bitstring>=3.1.5',
        'ipaddress>=1.0.18',
        'django-test-generator>=0.3.3',
        'mock',
        'coverage',
        'queryparser_python%d>=0.2.3' % sys.version_info.major
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Scientific/Engineering :: Astronomy'
    ],
    packages=find_packages(),
    include_package_data=True
)



from django.conf.urls import url, include

from rest_framework import routers

from .views import ManagementView, DatabaseView, TableView
from .viewsets import (
    DatabaseViewSet,
    TableViewSet,
    ColumnViewSet,
    FunctionViewSet,
    TableTypeViewSet,
    LicenseViewSet,
    AccessLevelViewSet
)

router = routers.DefaultRouter()
router.register(r'databases', DatabaseViewSet, base_name='database')
router.register(r'tables', TableViewSet, base_name='table')
router.register(r'columns', ColumnViewSet, base_name='column')
router.register(r'functions', FunctionViewSet, base_name='function')
router.register(r'tabletypes', TableTypeViewSet, base_name='tabletype')
router.register(r'licenses', LicenseViewSet, base_name='license')
router.register(r'accesslevels', AccessLevelViewSet, base_name='accesslevel')

urlpatterns = [
    url(r'^api/', include(router.urls)),

    url(r'^management/$', ManagementView.as_view(), name='management'),

    url(r'^(?P<database_name>\w+)/$', DatabaseView.as_view(), name='database'),
    url(r'^(?P<database_name>\w+)/(?P<table_name>\w+)/$', TableView.as_view(), name='table'),
]



from rest_framework import serializers


class JSONField(serializers.JSONField):

    def to_internal_value(self, data):
        if not isinstance(data, dict):
            self.fail('invalid')

        return super(JSONField, self).to_internal_value(data)


class ChoicesSerializer(serializers.Serializer):
    id = serializers.SerializerMethodField()
    text = serializers.SerializerMethodField()

    def get_id(self, obj):
        return obj[0]

    def get_text(self, obj):
        return obj[1]



from django.utils.xmlutils import SimplerXMLGenerator
from django.utils.six.moves import StringIO
from django.utils.encoding import smart_text

from rest_framework.renderers import BaseRenderer

from .utils import get_job_url

class XMLRenderer(BaseRenderer):

    media_type = 'application/xml'
    charset = 'utf-8'
    format = 'xml'

    def render(self, data, accepted_media_type=None, renderer_context=None):
        if data is None:
            return ''

        stream = StringIO()

        self.xml = SimplerXMLGenerator(stream, self.charset)
        self.xml.startDocument()
        self.render_document(data, accepted_media_type, renderer_context)
        self.xml.endDocument()
        return stream.getvalue()

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        raise NotImplementedError()

    def start(self, tag, attr={}):
        self.xml.startElement(tag, attr)

    def end(self, tag):
        self.xml.endElement(tag)

    def node(self, tag, attr, text):
        if not text:
            attr.update({'xsi:nil': 'true'})
        self.xml.startElement(tag, attr)
        if text:
            self.xml.characters(smart_text(text))
        self.xml.endElement(tag)

    def _to_camel_case(self, snake_str):
        components = snake_str.split('_')
        return components[0] + "".join(x.title() for x in components[1:])


class UWSRenderer(XMLRenderer):

    root_attrs = {
        'xmlns:uws': 'http://www.ivoa.net/xml/UWS/v1.0',
        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
        'xmlns:xlink': 'http://www.w3.org/1999/xlink',
        'version': '1.1'
    }

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        request = renderer_context['request']

        if isinstance(data, dict):
            if list(data.keys()) == ['results']:
                self.render_results(data['results'], request, root=True)
            elif list(data.keys()) == ['parameters']:
                self.render_parameters(data['parameters'], request, root=True)
            else:
                self.render_job(data, request)
        else:
            self.render_jobs(data, request)

    def render_jobs(self, data, request):
        self.start('uws:jobs', self.root_attrs)

        for item in data:
            self.start('uws:jobref', {
                'id': item['id'],
                'xlink:href': get_job_url(request, kwargs={'pk': item['id']}),
                'xlink:type': 'simple'
            })
            self.node('uws:phase', {}, item['phase'])
            self.end('uws:jobref')

        self.end('uws:jobs')

    def render_job(self, data, request):
        self.start('uws:job', self.root_attrs)

        for key, value in data.items():
            if key == 'results':
                self.render_results(value, request)

            elif key == 'parameters':
                self.render_parameters(value, request)

            else:
                self.node('uws:' + self._to_camel_case(key), {}, value)

        self.end('uws:job')

    def render_results(self, data, request, root=False):
        self.start('uws:results', self.root_attrs if root else {})

        for format_key, url in data.items():
            self.start('uws:result', {
                'id': format_key,
                'xlink:href': request.build_absolute_uri(url),
                'xlink:type': format_key
            })
            self.end('uws:result')

        self.end('uws:results')

    def render_parameters(self, data, request, root=False):
        self.start('uws:parameters', self.root_attrs if root else {})

        for parameter_key, parameter_value in data.items():
            self.node('uws:parameter', {'id': parameter_key}, parameter_value)

        self.end('uws:parameters')


class VOTableRenderer(XMLRenderer):

    media_type = 'application/x-votable+xml'
    format = 'votable'

    root_attrs = {
        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
        'xmlns': 'http://www.ivoa.net/xml/VOTable/v1.3',
        'xmlns:stc': 'http://www.ivoa.net/xml/STC/v1.30'
    }

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        self.start('VOTABLE', self.root_attrs)
        self.render_votable(data)
        self.end('VOTABLE')

    def render_votable(self, data, accepted_media_type=None, renderer_context=None):
        raise NotImplementedError()


class ErrorRenderer(VOTableRenderer):

    def get_error_string(self, data):
        if isinstance(data, dict):
            errors = []
            for parameter, parameter_errors in data.items():
                if isinstance(parameter_errors, (list, tuple)):
                    errors.append('%s: %s' % (parameter, ', '.join(parameter_errors)))
                else:
                    errors.append('%s: %s' % (parameter, parameter_errors))
            return '\n'.join(errors)
        elif isinstance(data, (list, tuple)):
            return '\n'.join(data)
        else:
            return data


    def render_votable(self, data, accepted_media_type=None, renderer_context=None):
        self.start('RESOURCE', {
            'type': 'results'
        })
        self.node('INFO', {
            'name': 'QUERY_STATUS',
            'value': 'ERROR'
        }, self.get_error_string(data))
        self.end('RESOURCE')



from django.conf.urls import url, include

from rest_framework import routers

from .views import table, archive
from .viewsets import RowViewSet, ColumnViewSet

router = routers.DefaultRouter()
router.register(r'rows', RowViewSet, base_name='row')
router.register(r'columns', ColumnViewSet, base_name='column')

urlpatterns = [
    url(r'^table/(?P<database_name>[A-Za-z0-9_]+)/(?P<table_name>[A-Za-z0-9_]+)/$', table, name='table'),
    url(r'^archive/(?P<database_name>[\w-]+)/(?P<table_name>[\w-]+)/(?P<column_name>[\w-]+)/$', archive, name='archive'),

    # rest api
    url(r'^api/', include(router.urls)),
]



from rest_framework import serializers

from daiquiri.jobs.models import Job

from .utils import make_query_dict_upper_case


class JobListSerializer(serializers.ModelSerializer):

    class Meta:
        model = Job
        fields = ('id', 'phase')


class JobRetrieveSerializer(serializers.ModelSerializer):

    job_id = serializers.UUIDField(source='id')
    owner_id = serializers.UUIDField(source='owner.username')
    destruction = serializers.DateTimeField(source='destruction_time')

    class Meta:
        model = Job
        fields = (
            'job_id',
            'owner_id',
            'phase',
            'quote',
            'start_time',
            'end_time',
            'execution_duration',
            'destruction',
            'results',
            'parameters'
        )


class CaseInsensitiveSerializer(serializers.Serializer):

    def __init__(self, *args, **kwargs):
        # make all the keys in kwargs['data'] upper case
        if 'data' in kwargs:
            kwargs['data'] = make_query_dict_upper_case(kwargs['data'])

        super(CaseInsensitiveSerializer, self).__init__(*args, **kwargs)


class JobUpdateSerializer(CaseInsensitiveSerializer):

    ACTION = serializers.CharField(required=False)
    DESTRUCTION = serializers.DateTimeField(required=False)
    EXECUTIONDURATION = serializers.IntegerField(required=False)
    PHASE = serializers.CharField(required=False)


class SyncJobSerializer(CaseInsensitiveSerializer):

    RESPONSEFORMAT = serializers.CharField(required=False)
    MAXREC = serializers.IntegerField(required=False)
    RUNID = serializers.CharField(required=False)


class AsyncJobSerializer(CaseInsensitiveSerializer):

    PHASE = serializers.CharField(required=False)
    RESPONSEFORMAT = serializers.CharField(required=False)
    MAXREC = serializers.IntegerField(required=False)
    RUNID = serializers.CharField(required=False)



import os

from .base import BASE_DIR

ASYNC = False

IPV4_PRIVACY_MASK = 16
IPV6_PRIVACY_MASK = 32

AUTH_WORKFLOW = None

AUTH_DETAIL_KEYS = []

QUERY_ANONYMOUS = False

QUERY_USER_DATABASE_PREFIX = 'daiquiri_user_'

QUERY_QUOTA = {
    'anonymous': '100Mb',
    'user': '100Mb',
    'users': {},
    'groups': {}
}

QUERY_QUEUES = [
    {
        'key': 'default',
        'label': 'Default',
        'timeout': 300,
        'priority': 1
    }
]

QUERY_LANGUAGES = [
    {
        'key': 'adql',
        'version': 2.0,
        'label': 'ADQL',
        'description': '',
        'quote_char': '"'
    }
]

QUERY_FORMS = [
    {
        'key': 'sql',
        'label': 'SQL query',
        'service': 'query/js/forms/sql.js',
        'template': 'query/query_form_sql.html'
    },
    {
        'key': 'box',
        'label': 'Box search',
        'service': 'query/js/forms/box.js',
        'template': 'query/query_form_box.html'
    },
    {
        'key': 'cone',
        'label': 'Cone search',
        'service': 'query/js/forms/cone.js',
        'template': 'query/query_form_cone.html'
    }
]

QUERY_DROPDOWNS = [
    {
        'key': 'simbad',
        'service': 'query/js/dropdowns/simbad.js',
        'template': 'query/query_dropdown_simbad.html',
        'options': {
            'url': 'http://simbad.u-strasbg.fr/simbad/sim-id'
        }
    },
    {
        'key': 'vizier',
        'service': 'query/js/dropdowns/vizier.js',
        'template': 'query/query_dropdown_vizier.html',
        'options': {
            'url': 'http://vizier.u-strasbg.fr/viz-bin/votable',
            'catalogs': ['I/322A', 'I/259']
        }
    }
]

QUERY_DOWNLOAD_DIR = os.path.join(BASE_DIR, 'download')
QUERY_DEFAULT_DOWNLOAD_FORMAT = 'votable'
QUERY_DOWNLOAD_FORMATS = [
    {
        'key': 'csv',
        'extension': 'csv',
        'content_type': 'text/csv',
        'label': 'Comma separated Values',
        'help': 'A text file with a line for each row of the table. The fields are delimited by a comma and quoted by double quotes. Use this option for a later import into a spreadsheed application or a custom script. Use this option if you are unsure what to use.'
    },
    {
        'key': 'votable',
        'extension': 'votable.xml',
        'content_type': 'application/xml',
        'label': 'IVOA VOTable XML file - TABLEDATA serialization',
        'help': 'A XML file using the IVOA VOTable format. Use this option if you intend to use VO compatible software to further process the data.'
    },
    {
        'key': 'votable-binary',
        'extension': 'votable.binary.xml',
        'content_type': 'application/xml',
        'label': 'IVOA VOTable XML file - BINARY serialization',
        'help': 'A XML file using the IVOA VOTable format (BINARY Serialization). Use this option if you intend to use VO compatible software to process the data and prefer the use of a binary file.'
    },
    {
        'key': 'votable-binary2',
        'extension': 'votable.binary2.xml',
        'content_type': 'application/xml',
        'label': 'IVOA VOTable XML file - BINARY 2 serialization',
        'help': 'A XML file using the IVOA VOTable format (BINARY2 Serialization). Use this option if you intend to use VO compatible software to process the data and prefer the use of a binary file.'
    }
]

SERVE_DOWNLOAD_DIR = os.path.join(BASE_DIR, 'download')

TAP_ACCESS_LEVEL = 'PUBLIC'

UWS_RESOURCES = []

WORDPRESS_URL = '/cms/'
WORDPRESS_CLI = '/opt/wp-cli/wp'
WORDPRESS_PATH = '/opt/wordpress'



from django.test import TestCase

from test_generator.views import TestListViewMixin


class TapViewTestCase(TestCase):

    fixtures = (
        'auth.json',
        'metadata.json',
        'examples.json'
    )

    users = (
        ('admin', 'admin'),
        ('user', 'user'),
        ('anonymous', None),
    )


class RootTests(TestListViewMixin, TapViewTestCase):

    url_names = {
        'list_view': 'tap:root'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }


class AvailabilityTests(TestListViewMixin, TapViewTestCase):

    url_names = {
        'list_view': 'tap:availability'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }


class CapabilitiesTests(TestListViewMixin, TapViewTestCase):

    url_names = {
        'list_view': 'tap:capabilities'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }


class TablesTests(TestListViewMixin, TapViewTestCase):

    url_names = {
        'list_view': 'tap:tables'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }


class ExamplesTests(TestListViewMixin, TapViewTestCase):

    url_names = {
        'list_view': 'tap:examples'
    }

    status_map = {
        'list_view': {
            'admin': 200, 'user': 200, 'anonymous': 200
        }
    }



from django.contrib import admin

from .models import QueryJob, Example


class QueryJobAdmin(admin.ModelAdmin):
    search_fields = ('id', 'job_type', 'owner__username', 'phase', 'database_name', 'table_name')
    list_display = ('id', 'job_type', 'owner', 'phase', 'creation_time', 'database_name', 'table_name', 'nrows')


class ExampleAdmin(admin.ModelAdmin):
    search_fields = ('name', 'query_string')
    list_display = ('order', 'name', 'query_string')
    list_display_links = ('name', )


admin.site.register(QueryJob, QueryJobAdmin)
admin.site.register(Example, ExampleAdmin)



import logging
import os

from celery.result import AsyncResult, EagerResult

from sendfile import sendfile

from django.conf import settings
from django.shortcuts import render
from django.http import HttpResponse, Http404

from .tasks import create_download_archive
from .utils import get_columns, get_archive_files, get_archive_file_name

logger = logging.getLogger(__name__)


def table(request, database_name, table_name):

    columns = get_columns(request.user, database_name, table_name)

    if columns:
        return render(request, 'serve/table.html', {
            'database': database_name,
            'table': table_name
        })

    # if nothing worked, return 404
    raise Http404


def archive(request, database_name, table_name, column_name):

    files = get_archive_files(request.user, database_name, table_name, column_name)

    if files:
        file_name = get_archive_file_name(request.user, table_name, column_name)

        task_id = file_name
        task_args = (file_name, files)

        if not settings.ASYNC:
            if os.path.isfile(file_name):
                task_result = EagerResult(task_id, None, 'SUCCESS')
            else:
                logger.info('create_download_archive %s submitted (sync)' % file_name)
                task_result = create_download_archive.apply(task_args, task_id=task_id, throw=True)

        else:
            task_result = AsyncResult(task_id)

            if not os.path.isfile(file_name):
                # create an empty file to prevent multiple pending tasks
                open(file_name, 'a').close()

                if task_result.successful():
                    # somebody or something removed the file. start all over again
                    task_result.forget()

                logger.info('create_download_archive %s submitted (async, queue=download)' % file_name)
                task_result = create_download_archive.apply_async(task_args, task_id=task_id)

        if task_result.successful():
            if request.method == 'GET':
                return sendfile(request, file_name, attachment=True)
            else:
                return HttpResponse(task_result.status)

        else:
            if task_result.status == 'FAILURE':
                return HttpResponse(task_result.status, status=500)
            else:
                return HttpResponse(task_result.status)

    # if nothing worked, return 404
    raise Http404



import os

from django import template
from django.conf import settings
from django.core.files import File
from django.utils.safestring import mark_safe

from daiquiri.core.utils import get_script_alias

register = template.Library()


@register.simple_tag(takes_context=True)
def wordpress_url(context, path=None):
    url = get_script_alias(context.request) + settings.WORDPRESS_URL

    if path:
        url += path

    return url


@register.simple_tag(takes_context=True)
def wordpress_admin_url(context):
    return get_script_alias(context.request) + settings.WORDPRESS_URL + 'wp-admin/'


@register.simple_tag()
def wordpress_menu(menu_name):
    try:
        with open(os.path.join(settings.WORDPRESS_PATH, 'wp-content', 'menus', menu_name + '.html')) as f:
            menu_file = File(f)
            return mark_safe(menu_file.read())
    except IOError:
        return ''



from django.conf import settings

from daiquiri.core.constants import ACCESS_LEVEL_PUBLIC

from .models import (
    Schema as TapSchema,
    Table as TapTable,
    Column as TapColumn,
)


def update_database(database):
    '''
    Update or create the database in the TAP_SCHEMA.
    '''
    # check if the metadata_access_level is public or it matches the TAP_ACCESS_LEVEL
    tap = (database.metadata_access_level == ACCESS_LEVEL_PUBLIC) or \
        (database.metadata_access_level == settings.TAP_ACCESS_LEVEL)

    if tap:
        try:
            tap_schema = TapSchema.objects.get(pk=database.id)
        except TapSchema.DoesNotExist:
            tap_schema = TapSchema.objects.create(pk=database.id)

        tap_schema.schema_name = database.name
        tap_schema.utype = None
        if database.description:
            tap_schema.description = database.description[:255]
        tap_schema.save()

    else:
        # remove the database from the TAP_SCHEMA (if it exists)
        try:
            TapSchema.objects.get(pk=database.id).delete()
        except TapSchema.DoesNotExist:
            pass

    # call the handler for each table of the database
    for table in database.tables.all():
        update_table(table)


def delete_database(database):
    '''
    Remove the database from the TAP_SCHEMA (if it exists).
    '''
    try:
        TapSchema.objects.get(pk=database.id).delete()
    except TapSchema.DoesNotExist:
        pass


def update_table(table):
    '''
    Update or create the table in the TAP_SCHEMA.
    '''
    # check if the metadata_access_level is public or it matches the TAP_ACCESS_LEVEL
    tap = (table.metadata_access_level == ACCESS_LEVEL_PUBLIC) or \
        (table.metadata_access_level == settings.TAP_ACCESS_LEVEL)

    # get the schema from the TAP_SCHEMA
    try:
        tap_schema = TapSchema.objects.get(pk=table.database.id)
    except TapSchema.DoesNotExist:
        tap_schema = None

    if tap and tap_schema:
        try:
            tap_table = TapTable.objects.get(pk=table.id)
            tap_table.schema = tap_schema
        except TapTable.DoesNotExist:
            tap_table = TapTable.objects.create(pk=table.id, schema=tap_schema)

        tap_table.schema_name = str(table.database)
        tap_table.table_name = table.name
        tap_table.table_type = table.type
        tap_table.utype = table.utype
        if table.description:
            tap_table.description = table.description[:255]
        tap_table.table_index = table.order

        tap_table.save()

    else:
        # remove the table from the TAP_SCHEMA (if it exists)
        try:
            TapTable.objects.get(pk=table.id).delete()
        except TapTable.DoesNotExist:
            pass

    # call the handler for each column of the table
    for column in table.columns.all():
        update_column(column)


def delete_table(table):
    '''
    Remove the table from the TAP_SCHEMA (if it exists).
    '''
    try:
        TapTable.objects.get(pk=table.id).delete()
    except TapTable.DoesNotExist:
        pass


def update_column(column):
    '''
    Update or create the column in the TAP_SCHEMA.
    '''
    # check if the metadata_access_level is public or it matches the TAP_ACCESS_LEVEL
    tap = (column.metadata_access_level == ACCESS_LEVEL_PUBLIC) or \
        (column.metadata_access_level == settings.TAP_ACCESS_LEVEL)

    # get the table from the TAP_SCHEMA
    try:
        tap_table = TapTable.objects.get(pk=column.table.id)
    except TapTable.DoesNotExist:
        tap_table = None

    if tap and tap_table:
        try:
            tap_column = TapColumn.objects.get(pk=column.id)
            tap_column.table = tap_table
        except TapColumn.DoesNotExist:
            tap_column = TapColumn.objects.create(pk=column.id, table=tap_table)

        tap_column.table_name = str(column.table)
        tap_column.column_name = column.name
        tap_column.datatype = column.datatype
        tap_column.arraysize = column.arraysize
        tap_column.size = column.arraysize
        if column.description:
            tap_column.description = column.description[:255]
        tap_column.utype = column.utype
        tap_column.unit = column.unit
        tap_column.ucd = column.ucd
        tap_column.indexed = column.indexed
        tap_column.principal = column.principal
        tap_column.std = column.std
        tap_column.column_index = column.order
        tap_column.save()
    else:
        # remove the column from the TAP_SCHEMA (if it exists)
        try:
            TapColumn.objects.get(pk=column.id).delete()
        except TapColumn.DoesNotExist:
            pass


def delete_column(column):
    '''
    Remove the column from the TAP_SCHEMA (if it exists).
    '''
    try:
        TapColumn.objects.get(pk=column.id).delete()
    except TapColumn.DoesNotExist:
        pass
